'From Pluralsight Domain-Driven Design Fundamentals Course (bit.ly/PS-DDD)

Imports System.Reflection

'this base class comes from Jimmy Bogard
'http://grabbagoft.blogspot.com/2007/06/generic-value-object-equality.html

Namespace MvcSalesApp.SharedKernel
  Public MustInherit Class ValueObject(Of T As ValueObject(Of T))
	  Implements IEquatable(Of T)

	Public Overridable Overloads Function Equals(ByVal other As T) As Boolean Implements IEquatable(Of T).Equals
	  If other Is Nothing Then
		Return False
	  End If

	  Dim t = Me.GetType
	  Dim otherType = other.GetType

	  If t IsNot otherType Then
		Return False
	  End If

	  Dim fields = t.GetFields(BindingFlags.Instance Or BindingFlags.NonPublic Or BindingFlags.Public)

	  For Each field In fields
		Dim value1 = field.GetValue(other)
		Dim value2 = field.GetValue(Me)

		If value1 Is Nothing Then
		  If value2 IsNot Nothing Then
			Return False
		  End If
		ElseIf Not value1.Equals(value2) Then
		  Return False
		End If
	  Next field

	  Return True
	End Function

	Public Overrides Overloads Function Equals(ByVal obj As Object) As Boolean
	  If obj Is Nothing Then
		Return False
	  End If

	  Dim other = TryCast(obj, T)

	  Return Equals(other)
	End Function

	Public Overrides Function GetHashCode() As Integer
	  Dim fields = GetFields()

	  Dim startValue = 17
	  Dim multiplier = 59

	  Dim hashCode = startValue

	  For Each field In fields
		Dim value = field.GetValue(Me)

		If value IsNot Nothing Then
		  hashCode = hashCode*multiplier + value.GetHashCode
		End If
	  Next field

	  Return hashCode
	End Function

	Private Function GetFields() As IEnumerable(Of FieldInfo)
	  Dim t = Me.GetType

	  Dim fields = New List(Of FieldInfo)

	  Do While t IsNot GetType(Object)
		fields.AddRange(t.GetFields(BindingFlags.Instance Or BindingFlags.NonPublic Or BindingFlags.Public))

		t = t.BaseType
	  Loop

	  Return fields
	End Function

	Public Shared Operator =(ByVal x As ValueObject(Of T), ByVal y As ValueObject(Of T)) As Boolean
	  Return x.Equals(y)
	End Operator

	Public Shared Operator <>(ByVal x As ValueObject(Of T), ByVal y As ValueObject(Of T)) As Boolean
	  Return Not(x = y)
	End Operator
  End Class
End Namespace
