Imports MvcSalesApp.SharedKernel
Imports Order.Domain
Imports Order.Domain.ValueObjects
Imports Order.Write.Domain.DTOs

Namespace Order.Write.Domain
  Public Class SalesOrder
	  Inherits Entity

	Private ReadOnly _customer As Customer
	Private ReadOnly _lineItems As List(Of LineItem)

	Public Shared Function Create(ByVal cartItems As IEnumerable(Of CartItem), ByVal customer As Customer) As SalesOrder
	  Dim order = New SalesOrder(cartItems, customer)
	  Return order
	End Function

	Private Sub New(ByVal cartItems As IEnumerable(Of CartItem), ByVal customer As Customer)
		Me.New()
	  Id = Guid.NewGuid
	  _customer = customer
	  CustomerId = customer.CustomerId
	  SetShippingAddress(customer.PrimaryAddress)
	  ApplyCustomerStatusDiscount()
	  For Each item In cartItems
		CreateLineItem(item.ProductId, CDbl(item.Price), item.Quantity)
	  Next item
	  _customer = customer
	End Sub

	Protected Sub New()
	  _lineItems = New List(Of LineItem)
	  Id = Guid.NewGuid
	  OrderDate = Date.Now
	End Sub

	Private privateOrderDate As Date
	Public Property OrderDate As Date
		Get
			Return privateOrderDate
		End Get
		Private Set(ByVal value As Date)
			privateOrderDate = value
		End Set
	End Property

	Private privateDueDate? As Date
	Public Property DueDate As Date?
		Get
			Return privateDueDate
		End Get
		Private Set(ByVal value? As Date)
			privateDueDate = value
		End Set
	End Property

	Private privateOnlineOrder As Boolean
	Public Property OnlineOrder As Boolean
		Get
			Return privateOnlineOrder
		End Get
		Private Set(ByVal value As Boolean)
			privateOnlineOrder = value
		End Set
	End Property

	Private privatePurchaseOrderNumber As String
	Public Property PurchaseOrderNumber As String
		Get
			Return privatePurchaseOrderNumber
		End Get
		Private Set(ByVal value As String)
			privatePurchaseOrderNumber = value
		End Set
	End Property

	Private privateComment As String
	Public Property Comment As String
		Get
			Return privateComment
		End Get
		Private Set(ByVal value As String)
			privateComment = value
		End Set
	End Property

	Private privatePromotionId As Integer
	Public Property PromotionId As Integer
		Get
			Return privatePromotionId
		End Get
		Private Set(ByVal value As Integer)
			privatePromotionId = value
		End Set
	End Property

	Private privateShippingAddress As Address
	Public Property ShippingAddress As Address
		Get
			Return privateShippingAddress
		End Get
		Private Set(ByVal value As Address)
			privateShippingAddress = value
		End Set
	End Property

	Private privateCurrentCustomerStatus As CustomerStatus
	Public Property CurrentCustomerStatus As CustomerStatus
		Get
			Return privateCurrentCustomerStatus
		End Get
		Private Set(ByVal value As CustomerStatus)
			privateCurrentCustomerStatus = value
		End Set
	End Property

	Public ReadOnly Property Discount As Double
	  Get
		  Return CustomerDiscount + PromoDiscount
	  End Get
	End Property

	Private privateCustomerDiscount As Double
	Public Property CustomerDiscount As Double
		Get
			Return privateCustomerDiscount
		End Get
		Private Set(ByVal value As Double)
			privateCustomerDiscount = value
		End Set
	End Property

	Private privatePromoDiscount As Double
	Public Property PromoDiscount As Double
		Get
			Return privatePromoDiscount
		End Get
		Private Set(ByVal value As Double)
			privatePromoDiscount = value
		End Set
	End Property

	Private privateSalesOrderNumber As String
	Public Property SalesOrderNumber As String
		Get
			Return privateSalesOrderNumber
		End Get
		Private Set(ByVal value As String)
			privateSalesOrderNumber = value
		End Set
	End Property

	Private privateCustomerId As Integer
	Public Property CustomerId As Integer
		Get
			Return privateCustomerId
		End Get
		Private Set(ByVal value As Integer)
			privateCustomerId = value
		End Set
	End Property

	Private privateSubTotal As Double
	Public Property SubTotal As Double
		Get
			Return privateSubTotal
		End Get
		Private Set(ByVal value As Double)
			privateSubTotal = value
		End Set
	End Property

	Public ReadOnly Property LineItems As ICollection(Of LineItem)
	  Get
		  Return _lineItems
	  End Get
	End Property

	Public Sub CreateLineItem(ByVal productId As Integer, ByVal listPrice As Double, ByVal quantity As Integer)
	  'NOTE: add some validations such as:
	  'quantity can't be 0 and if it's >1000 give a discount
	  ' customer is a gold. give another discount
	  Dim item = LineItem.Create(Id, productId, quantity, listPrice, CustomerDiscount + PromoDiscount)
	  _lineItems.Add(item)
	End Sub

	Public Sub SetShippingAddress(ByVal address As Address)
	  ShippingAddress = Address.Create(address.Street, address.City, address.StateProvince, address.PostalCode)
	End Sub

	Public Function HasLineItems() As Boolean
	  If LineItems.Any Then
		Return True
	  End If
	  Return False
	End Function

	Public Function CalculateShippingCost() As Decimal
	  'items, quantity, price,disounts, total weight of item
	  'this is the job of a microservice we can call out to
	  Throw New NotImplementedException
	End Function

	Public Sub ApplyCustomerStatusDiscount()
	  Dim status = CustomerStatus.New
	  If _customer IsNot Nothing Then
		status = _customer.Status
	  End If
	  Select Case status
		Case CustomerStatus.Silver
		  CustomerDiscount = CustomerDiscounts.SilverDiscount

		Case CustomerStatus.Gold
		  CustomerDiscount = CustomerDiscounts.GoldDiscount

		Case CustomerStatus.Platinum
		  CustomerDiscount = CustomerDiscounts.PlatinumDiscount

		Case Else
		  CustomerDiscount = 0
		  Exit Select
	  End Select
	End Sub

	Public Sub SetOrderDetails(ByVal onLineOrder As Boolean, ByVal PONumber As String, ByVal comment As String, ByVal promotionId As Integer, ByVal promoDiscount As Double)
	  Me.OnlineOrder = onLineOrder
	  PurchaseOrderNumber = PONumber
	  Me.Comment = comment

	  Me.PromotionId = promotionId
	  Me.PromoDiscount = promoDiscount
	End Sub
  End Class
End Namespace
